###############   PSQF 6249 FA & SEM Example 4 CFA using R   #############

# Set width of output and  number of digits to be printed
options("width"=140, "digits"=4) 

# Use this to turn off scientific notation: 
# "fixed notation will be preferred unless it is more than 'scipen' digits wide"
options(scipen=9)

# turn off stars for different alpha levels
options(show.signif.stars=FALSE)

#####  Check to see if packages are downloaded, install if not, then load  #####

if (!require("readxl")) install.packages("readxl")
library(readxl) # To import xls or xlsx data as table

if (!require("expss")) install.packages("expss")
library(expss) # To add variable and value labels, sort

if (!require("TeachingDemos")) install.packages("TeachingDemos")
library(TeachingDemos) # To create text output files

if (!require("psych")) install.packages("psych")
library(psych) # To add summary functions

if (!require("ggplot2")) install.packages("ggplot2")
library(ggplot2) # To make pictures

if (!require("lavaan")) install.packages("lavaan")
library(lavaan) # To fit SEMs

# Other packages not used here, but which are pry useful

if (!require("semTools")) install.packages("semTools")
library(semTools) # lavaan add-ons (multiple imputation, moderation)

if (!require("MplusAutomation")) install.packages("MplusAutomation")
library(lavaan) # To use Mplus within R

#######################################################################################

# Define variables for working directory and data name
filesave = "C:\\Dropbox/22_PSQF6249/HFS/PSQF6249_Example4/R/"
filename = "Example4.xlsx"
setwd(dir=filesave)

# Import HFS item-level data
Example4 = read_excel(paste0(filesave,filename)) 
# Convert to data frame as data frame to use for analysis
Example4 = as.data.frame(Example4)

# Open external file to save results to
txtStart(file=paste0(filesave,"R_Example4_CFA_Output.txt"))

#### General notes about lavaan syntax objects:
# Once factors are defined they can be used in syntax as if they were observed variables
# Multiple commands can go on a single line separated by ; 
# Parameters can be fixed to constant values by using the * command
# Start values can be specified by using value* in front of parameter 
# Labels can be implemented with (label)* in front of parameter

# Example of how to concatenate syntax strings into one model
#Model <- sem(model=c(syntax1, syntax2), data = myData)

########################################################################################
# Model 1: Single Factor Using Fully Z-Scored Factor Scaling
# Factor Variance = 1, Factor Mean = 0, so all item loadings and intercepts estimated

# R Syntax for lavaan function: longer but more transparent version of model
Syntax1 = "
# Define factor and request item factor loadings --> factor =~ item + item + item
  Sit =~ Sit1r + Sit2 + Sit3r + Sit4 + Sit5r + Sit6
# Item intercepts --> ~ 1 indicates means or intercepts
  Sit1r ~ 1; Sit2 ~ 1; Sit3r ~ 1; Sit4 ~ 1; Sit5r ~ 1; Sit6 ~ 1
# Item error variances or covariances --> use the ~~ command
  Sit1r ~~ Sit1r; Sit2 ~~ Sit2; Sit3r ~~ Sit3r; Sit4 ~~ Sit4; Sit5r ~~ Sit5r; Sit6 ~~ Sit6 
# Factor variance fixed=1 and factor mean fixed=0 (* means fixed in lavaan)
  Sit ~~ 1*Sit; Sit ~ 0
"
# Use MLR estimation like in Mplus, z-scored latent variables (mean=0, SD=1)
  Model1 = lavaan(model=Syntax1, data=Example4, estimator="MLR", mimic="mplus", std.lv=TRUE)
# Print solution: get fit, get effect size, STDYX solution
  summary(object=Model1, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE)
# Get model-implied means, variances, and covariances
  fitted(object=Model1)
# Get residual (discrepancy) info for local misfit in covariance and correlation form
  resid(object=Model1, type="raw"); resid(object=Model1, type="cor")
# Get normalized residuals (like z-scores for discrepancies)
  resid(object=Model1, type="normalized")
# Get "cheat code" modification indices
  modificationindices(object = Model1, sort.=TRUE)

txtComment(" ") # insert blank space in output
  
# R Syntax for sem or cfa functions: shorter but less transparent version of model
  Syntax1short = "Sit =~ Sit1r + Sit2 + Sit3r + Sit4 + Sit5r + Sit6"
# Use MLR estimation like in Mplus, z-score latent factor (mean=0, SD=1)
  Model1short = sem(model=Syntax1short, data=Example4, estimator="MLR", mimic="mplus", std.lv=TRUE)
# Print solution: get fit, get effect size, STDYX solution (will be same as from lavaan)
  summary(object=Model1short, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE)

txtComment(" ") # insert blank space in output
  
  
########################################################################################
# Model 2: Single Factor Scaled Using Marker Item Loading=1 and Factor Mean=0
# Factor variance estimated given marker item loading=1, all intercepts estimated
  
# R Syntax for lavaan function: longer but more transparent version of model
  Syntax2 = "
# Define factor and request item factor loadings with first loading fixed to 1
  Sit =~ 1*Sit1r + Sit2 + Sit3r + Sit4 + Sit5r + Sit6
# Item intercepts all estimated
  Sit1r ~ 1; Sit2 ~ 1; Sit3r ~ 1; Sit4 ~ 1; Sit5r ~ 1; Sit6 ~ 1
# Item error variances all estimated
  Sit1r ~~ Sit1r; Sit2 ~~ Sit2; Sit3r ~~ Sit3r; Sit4 ~~ Sit4; Sit5r ~~ Sit5r; Sit6 ~~ Sit6 
# Factor variance=estimated and factor mean=0
  Sit ~~ Sit; Sit ~ 0
"
# Use MLR estimation like in Mplus, do not z-score latent factor (because variance is estimated)
  Model2 = lavaan(model=Syntax2, data=Example4, estimator="MLR", mimic="mplus", std.lv=FALSE)
# Print solution: get fit, get effect size, STDYX solution
  summary(object=Model2, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE)

txtComment(" ") # insert blank space in output
  
# R Syntax for sem or cfa functions: shorter but less transparent version of model
  Syntax2short = "Sit =~ 1*Sit1r + Sit2 + Sit3r + Sit4 + Sit5r + Sit6"
# Use MLR estimation like in Mplus, do not z-score latent factor (because variance is estimated)
  Model2short = sem(model=Syntax2short, data=Example4, estimator="MLR", mimic="mplus", std.lv=FALSE)
# Print solution: get fit, get effect size, STDYX solution (will be same as from lavaan)
  summary(object=Model2short, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE)
  
txtComment(" ") # insert blank space in output
  
  
########################################################################################
# Model 3: Single Factor Scaled Using Marker Item Loading=1 and Intercept=0
# Factor variance and mean estimated given marker item loading=1 and intercept=0
  
# R Syntax for lavaan function: longer but more transparent version of model
  Syntax3 = "
# Define factor and request item factor loadings with first loading fixed to 1
  Sit =~ 1*Sit1r + Sit2 + Sit3r + Sit4 + Sit5r + Sit6
# Item intercepts: first is fixed to 0 and rest estimated
  Sit1r ~ 0; Sit2 ~ 1; Sit3r ~ 1; Sit4 ~ 1; Sit5r ~ 1; Sit6 ~ 1
# Item error variances all estimated
  Sit1r ~~ Sit1r; Sit2 ~~ Sit2; Sit3r ~~ Sit3r; Sit4 ~~ Sit4; Sit5r ~~ Sit5r; Sit6 ~~ Sit6 
# Factor variance=estimated and factor mean=estimated
  Sit ~~ Sit; Sit ~ 1
"
# Use MLR estimation like in Mplus, do not z-score latent factor (mean and variance estimated)
  Model3 = lavaan(model=Syntax3, data=Example4, estimator="MLR", mimic="mplus", std.lv=FALSE)
# Print solution: get fit, get effect size, STDYX solution
  summary(object=Model3, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE)
  
# No short version possible using sem that I know of
  
txtComment(" ") # insert blank space in output
  

########################################################################################
# Saturated Model for Demonstration Purposes
# All item means, variances, and covariances estimated
  
# R Syntax for lavaan function: longer but more transparent version of model
  SyntaxSat = "
# Item means all estimated
  Sit1r ~ 1; Sit2 ~ 1; Sit3r  ~ 1; Sit4 ~ 1; Sit5r ~ 1; Sit6 ~ 1
# Item variances all estimated
  Sit1r ~~ Sit1r; Sit2 ~~ Sit2; Sit3r ~~ Sit3r; Sit4 ~~ Sit4; Sit5r ~~ Sit5r; Sit6 ~~ Sit6 
# Covariances all estimated
  Sit1r ~~ Sit2; Sit1r ~~ Sit3r; Sit1r ~~ Sit4; Sit1r ~~ Sit5r; Sit1r ~~ Sit6 
  Sit2 ~~ Sit3r; Sit2 ~~ Sit4; Sit2 ~~ Sit5r; Sit2 ~~ Sit6
  Sit3r ~~ Sit4; Sit3r ~~ Sit5r; Sit3r ~~ Sit6
  Sit4 ~~ Sit5r; Sit4 ~~ Sit6; Sit5r ~~ Sit6
"
ModelSat = lavaan(model=SyntaxSat, data=Example4, estimator="MLR", mimic="mplus")
summary(ModelSat, fit.measures=TRUE, standardized=TRUE)
# Get saturated model-implied means, variances, and covariances
fitted(object=ModelSat)

# No short version possible using sem that I know of

txtComment(" ") # insert blank space in output


########################################################################################
# Null Model for Demonstration Purposes
# All item means and variances estimated, but all covariances fixed=0

# R Syntax for lavaan function: longer but more transparent version of model
  SyntaxNull = "
# Item means all estimated
  Sit1r ~ 1; Sit2 ~ 1; Sit3r  ~ 1; Sit4 ~ 1; Sit5r ~ 1; Sit6 ~ 1
# Item variances all estimated
  Sit1r ~~ Sit1r; Sit2 ~~ Sit2; Sit3r ~~ Sit3r; Sit4 ~~ Sit4; Sit5r ~~ Sit5r; Sit6 ~~ Sit6 
"
ModelNull = lavaan(model=SyntaxNull, data=Example4, estimator="MLR", mimic="mplus")
summary(ModelNull, fit.measures=TRUE)
# Get null model-implied means, variances, and covariances
fitted(object=ModelNull)

# No short version possible using sem that I know of

txtComment(" ") # insert blank space in output


############################################################################################
# Model 4: Two Factors Using Fully Z-Scored Factor Scaling (with labels at parameter front)
# Factor Variances = 1, Factor Means = 0, so all item loadings and intercepts estimated

# R Syntax for lavaan function: longer but more transparent version of model
Syntax4 = "  ! Note labels are in order of inclusion, not according to item name
# Define factor and request item factor loadings all estimated
  SitP =~ L1*Sit2  + L2*Sit4  + L3*Sit6     # Pos items
  SitN =~ L4*Sit1r + L5*Sit3r + L6*Sit5r    # Neg items
# Item intercepts all estimated
  Sit2  ~ I1*1; Sit4  ~ I2*1; Sit6  ~ I3*1  # Pos items
  Sit1r ~ I4*1; Sit3r ~ I5*1; Sit5r ~ I6*1  # Neg items
# Item error variances all estimated
  Sit2  ~~ E1*Sit2;  Sit4  ~~ E2*Sit4;  Sit6  ~~ E3*Sit6   # Pos items
  Sit1r ~~ E4*Sit1r; Sit3r ~~ E5*Sit3r; Sit5r ~~ E6*Sit5r  # Neg items
# Factor variances fixed=1 and factor means fixed=0 (won't allow labels)
  SitP ~~ 1*SitP; SitN ~~ 1*SitN; SitP ~ 0; SitN ~ 0
# Factor covariance estimated (and labeled)
  SitP ~~ FactCov*SitN
# Calculate Omega Reliability for Sum Score Per Factor (1=factor variance):
  OmegaP := (1*(L1 + L2 + L3)^2) / ( (1*(L1 + L2 + L3)^2) + (E1 + E2 + E3))
  OmegaN := (1*(L4 + L5 + L6)^2) / ( (1*(L4 + L5 + L6)^2) + (E4 + E5 + E6))
# Don't forget to include any error covariances, too! (Lecture 4 slide 77)
"
# Use MLR estimation like in Mplus, z-score latent factors (mean=0, SD=1)
Model4 = lavaan(model=Syntax4, data=Example4, estimator="MLR", mimic="mplus", std.lv=TRUE)
# Print solution: get fit, get effect size, STDYX solution
summary(object=Model4, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE)

print("LRT for one factor vs two factors: is correlation < 1?")
anova(Model1, Model4)

# Get model-implied means, variances, and covariances
fitted(object=Model4)
# Get residual (discrepancy) info for local misfit in covariance and correlation form
resid(object=Model4, type="raw"); resid(object=Model4, type="cor")
# Get normalized residuals (like z-scores for discrepancies)
resid(object=Model4, type="normalized")
# Get "cheat code" modification indices
modificationindices(object = Model4, sort.=TRUE)

# Get factor scores (without their SEs, though)
Model4Scores = lavPredict(object=Model4, newdata=NULL, type="lv", method="EBM",
                          se="standard", acov="standard", label=TRUE, fsm=FALSE, append.data=TRUE)
summary(Model4Scores)

txtComment(" ") # insert blank space in output

# R Syntax for sem or cfa functions: shorter but less transparent version of model
Syntax4short = "
  SitP =~ L1*Sit2  + L2*Sit4  + L3*Sit6     # Pos items
  SitN =~ L4*Sit1r + L5*Sit3r + L6*Sit5r    # Neg items
"
# Use MLR estimation like in Mplus, z-score latent factor (mean=0, SD=1)
Model4short = sem(model=Syntax4short, data=Example4, estimator="MLR", mimic="mplus", std.lv=TRUE)
# Print solution: get fit, get effect size, STDYX solution (will be same as from lavaan)
summary(object=Model4short, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE)

txtComment(" ") # insert blank space in output


##############################################################################################################
# Function written by Jonathan Templin to get factor scores and their SEs
# Available here: https://jonathantemplin.com/wp-content/uploads/2017/09/EPSY906_Example04.nb_.html#
factorScores = function(lavObject){
  output = inspect(object = lavObject, what = "est")
  
  sigma = output$lambda %*% output$psi %*% t(output$lambda) + output$theta
  modelData = lavObject@Data@X[[1]]
  
  scores = t(output$alpha%*%matrix(1, nrow=1, ncol=dim(modelData)[1]) +
             output$psi %*% t(output$lambda) %*% solve(sigma)%*%(t(modelData) - output$nu%*%matrix(1, nrow=1, ncol=dim(modelData)[1])))
  varscores = output$psi - output$psi %*% t(output$lambda) %*% solve(sigma) %*% output$lambda %*% output$psi
  factorSE = sqrt(diag(varscores))
  names(factorSE) = paste0(names(factorSE), ".SE")
  factorSEmat = matrix(1, nrow=nrow(scores), ncol = 1) %*% matrix(factorSE, nrow = 1, ncol = ncol(scores))
  colnames(factorSEmat) = names(factorSE)
  
  result = data.frame(cbind(scores, factorSEmat))
  names(result)
  odds = seq(1, ncol(result)-1, 2)
  evens = seq(2, ncol(result), 2)
  result = result[c(odds,evens)]
  
  factorCov = varscores
  factorCorr = solve(sqrt(diag(diag(varscores)))) %*% varscores %*% solve(sqrt(diag(diag(varscores))))
  
  return(list(scores = result, factorCov = factorCov, factorCorr = factorCorr))
}
factorScoreReliability = function(lavObject){
  output = inspect(object = lavObject, what = "est")
  sigma = output$lambda %*% output$psi %*% t(output$lambda) + output$theta
  varscores = output$psi - output$psi %*% t(output$lambda) %*% solve(sigma) %*% output$lambda %*% output$psi
  
  return(diag(output$psi)/(diag(output$psi) + diag(varscores)))
}
###############################################################################################################
# Call function to get factor scores from Model 4
factorScoreReliability(lavObject=Model4)

# calculate factor scores using function above
fscores = factorScores(lavObject=Model4)
# Show variance of factor scores
var(fscores$scores$SitP); var(fscores$scores$SitN)

# Histogram overlaid with kernel density curve
ggplot(fscores$scores, aes(x=SitP)) + 
  geom_histogram(aes(y=..density..), # Histogram with density instead of count on y-axis
                 binwidth=.5,
                 colour="black", fill="white") + xlim(c(-4,4)) +
                 labs(title = "Positive Situation Forgiveness Factor Score") +
  geom_density(alpha=.2, fill="#FF6666")    # Overlay with transparent density plot

ggplot(fscores$scores, aes(x=SitN)) + 
  geom_histogram(aes(y=..density..), # Histogram with density instead of count on y-axis
                 binwidth=.5,
                 colour="black", fill="white") + xlim(c(-4,4)) +
  labs(title = "Negative Situation Forgiveness Factor Score") +
  geom_density(alpha=.2, fill="#FF6666")    # Overlay with transparent density plot

########################################################################################
# Model 5: Two Factors with Tau-Equivalent Negative Items Only (loadings equal)

# R Syntax for sem function: shorter but less transparent version of model
Syntax5 = "  ! Note labels are in order of inclusion, not according to item name
# Define factor and request item factor loadings all estimated
  SitP =~ L1*Sit2 + L2*Sit4 + L3*Sit6                      # Pos items
  SitN =~ NegLoad*Sit1r + NegLoad*Sit3r + NegLoad*Sit5r    # Neg items
"
# Use MLR estimation like in Mplus, z-score latent factors (mean=0, SD=1)
Model5 = sem(model=Syntax5, data=Example4, estimator="MLR", mimic="mplus", std.lv=TRUE)
# Print solution: get fit, get effect size, STDYX solution
summary(object=Model5, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE)

print("LRT: Does the shared loading for the negative items make fit worse?")
anova(Model5, Model4)

txtComment(" ") # insert blank space in output


########################################################################################
# Model 6: Two Factors with Tau-Equivalent Positive Items Only (loadings equal)

# R Syntax for sem function: shorter but less transparent version of model
Syntax6 = "  ! Note labels are in order of inclusion, not according to item name
# Define factor and request item factor loadings all estimated
  SitP =~ PosLoad*Sit2 + PosLoad*Sit4 + PosLoad*Sit6       # Pos items
  SitN =~ L4*Sit1r + L5*Sit3r + L6*Sit5r                   # Neg items
# Item error variances all estimated
  Sit2  ~~ E1*Sit2;  Sit4  ~~ E2*Sit4;  Sit6  ~~ E3*Sit6   # Pos items
  Sit1r ~~ E4*Sit1r; Sit3r ~~ E5*Sit3r; Sit5r ~~ E6*Sit5r  # Neg items
# Calculate Alpha Reliability for Sum Score of Positive Factor:
  AlphaP := ((PosLoad*3)^2) / ( ((PosLoad*3)^2) + (E1 + E2 + E3))
"
# Use MLR estimation like in Mplus, z-score latent factors (mean=0, SD=1)
Model6 = sem(model=Syntax6, data=Example4, estimator="MLR", mimic="mplus", std.lv=TRUE)
# Print solution: get fit, get effect size, STDYX solution
summary(object=Model6, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE)

print("LRT: Does the shared loading for the positive items make fit worse?")
anova(Model6, Model4)

txtComment(" ") # insert blank space in output


#############################################################################################
# Model 7: Two Factors with Parallel Positive Items Only (loadings and error variances equal)

# R Syntax for sem function: shorter but less transparent version of model
Syntax7 = "  ! Note labels are in order of inclusion, not according to item name
# Define factor and request item factor loadings all estimated
  SitP =~ PosLoad*Sit2  + PosLoad*Sit4  + PosLoad*Sit6     # Pos items
  SitN =~ L4*Sit1r + L5*Sit3r + L6*Sit5r                   # Neg items
# Item error variances all estimated
  Sit2  ~~ PosError*Sit2; Sit4  ~~ PosError*Sit4; Sit6  ~~ PosError*Sit6   # Pos items
  Sit1r ~~ E4*Sit1r; Sit3r ~~ E5*Sit3r; Sit5r ~~ E6*Sit5r                  # Neg items
# Calculate Spearman-Brown Reliability for Sum Score of Positive Factor:
  SpearP := ((PosLoad*3)^2) / ( ((PosLoad*3)^2) + (PosError*3) )
"
# Use MLR estimation like in Mplus, z-score latent factors (mean=0, SD=1)
Model7 = sem(model=Syntax7, data=Example4, estimator="MLR", mimic="mplus", std.lv=TRUE)
# Print solution: get fit, get effect size, STDYX solution
summary(object=Model7, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE)

print("LRT: Does the shared error variance for the positive items make fit worse?")
anova(Model7, Model6)

txtComment(" ") # insert blank space in output


#############################################################################################

" # Example of using a factor score as a single indicator
Factor =~ 1*fscore; fscore ~~ ResVar*fscore; Factor ~~ Factor;
fscore ~ 1; Factor ~ 0; # Mean center factor
ResVar := (1-reliability)*(total variance)
"

# Close external file of saved results
txtStop()


